@@ -32,11 +32,14 @@ from copy import deepcopy as copy
 
 
 class ArbitraryOrderLagrange:
-    def __init__(self, order, ndim):
+    "Lagrange quad/hexahdron finite element of arbitrary order."
+
+    def __init__(self, order, ndim, interval=(-1, 1)):
         self.ndim = ndim
         self.order = order
-        self.nnodes = (order + 1) ** ndim
-        self.nbasis = self.nnodes
+        self.npoints = (order + 1) ** ndim
+        self.nbasis = self.npoints
+        self.interval = interval
 
         self._nbasis = order + 1
 
@@ -84,8 +87,7 @@ class ArbitraryOrderLagrange:
 
     def _points(self, n):
         "Equidistant n points in interval [-1, 1]."
-        i = np.arange(n)
-        return 2 * i / (n - 1) - 1
+        return np.linspace(*self.interval, n)
 
     def _polynomial(self, r, n):
         "Lagrange-Polynomial of order n evaluated at coordinate r."
@@ -93,35 +95,35 @@ class ArbitraryOrderLagrange:
         return r ** m / factorial(m)
 
 
-class Line:
+class LineElement:
     def __init__(self):
         self.ndim = 1
 
 
-class Quad:
+class QuadElement:
     def __init__(self):
         self.ndim = 2
 
 
-class Hex:
+class HexahedronElement:
     def __init__(self):
         self.ndim = 3
 
 
-class Tri:
+class TriangleElement:
     def __init__(self):
         self.ndim = 2
 
 
-class Tet:
+class TetraElement:
     def __init__(self):
         self.ndim = 3
 
 
-class Line1(Line):
+class Line(LineElement):
     def __init__(self):
         super().__init__()
-        self.nnodes = 2
+        self.npoints = 2
         self.nbasis = 2
 
     def basis(self, rv):
@@ -135,10 +137,10 @@ class Line1(Line):
         return np.array([[-1], [1]]) * 0.5
 
 
-class Quad0(Quad):
+class ConstantQuad(QuadElement):
     def __init__(self):
         super().__init__()
-        self.nnodes = 4
+        self.npoints = 4
         self.nbasis = 1
 
     def basis(self, rst):
@@ -150,10 +152,10 @@ class Quad0(Quad):
         return np.array([[0, 0, 0]])
 
 
-class Quad1(Quad):
+class Quad(QuadElement):
     def __init__(self):
         super().__init__()
-        self.nnodes = 4
+        self.npoints = 4
         self.nbasis = 4
 
     def basis(self, rs):
@@ -187,10 +189,10 @@ class Quad1(Quad):
         )
 
 
-class Hex0(Hex):
+class ConstantHexahedron(HexahedronElement):
     def __init__(self):
         super().__init__()
-        self.nnodes = 8
+        self.npoints = 8
         self.nbasis = 1
 
     def basis(self, rst):
@@ -202,10 +204,10 @@ class Hex0(Hex):
         return np.array([[0, 0, 0]])
 
 
-class Hex1(Hex):
+class Hexahedron(HexahedronElement):
     def __init__(self):
         super().__init__()
-        self.nnodes = 8
+        self.npoints = 8
         self.nbasis = 8
 
     def basis(self, rst):
@@ -247,10 +249,10 @@ class Hex1(Hex):
         )
 
 
-class Hex2s(Hex):
+class QuadraticHexahedron(HexahedronElement):
     def __init__(self):
         super().__init__()
-        self.nnodes = 20
+        self.npoints = 20
         self.nbasis = 20
 
     def basis(self, rst):
@@ -288,147 +290,165 @@ class Hex2s(Hex):
     def basisprime(self, rst):
         "quadratic serendipity hexahedron derivative of basis functions"
         r, s, t = rst
-        return (
-            np.array(
+        return np.array(
+            [
                 [
-                    [
-                        (1 - s) * (1 - t) * (2 + r + s + t) * 0.125
-                        - (1 - r) * (1 - s) * (1 - t) * (2 + s + t) * 0.125,
-                        (1 - r) * (1 - t) * (2 + r + s + t) * 0.125
-                        - (1 - r) * (1 - s) * (1 - t) * (2 + r + t) * 0.125,
-                        (1 - r) * (1 - s) * (2 + r + s + t) * 0.125
-                        - (1 - r) * (1 - s) * (1 - t) * (2 + r + s) * 0.125,
-                    ],
-                    [
-                        -(1 - s) * (1 - t) * (2 - r + s + t) * 0.125
-                        + (1 + r) * (1 - s) * (1 - t) * (2 + s + t) * 0.125,
-                        (1 + r) * (1 - t) * (2 - r + s + t) * 0.125
-                        - (1 + r) * (1 - s) * (1 - t) * (2 - r + t) * 0.125,
-                        (1 + r) * (1 - s) * (2 - r + s + t) * 0.125
-                        - (1 + r) * (1 - s) * (1 - t) * (2 - r + s) * 0.125,
-                    ],
-                    [
-                        -(1 + s) * (1 - t) * (2 - r - s + t) * 0.125
-                        + (1 + r) * (1 + s) * (1 - t) * (2 - s + t) * 0.125,
-                        -(1 + r) * (1 - t) * (2 - r - s + t) * 0.125
-                        + (1 + r) * (1 + s) * (1 - t) * (2 - r + t) * 0.125,
-                        (1 + r) * (1 + s) * (2 - r - s + t) * 0.125
-                        - (1 + r) * (1 + s) * (1 - t) * (2 - r - s) * 0.125,
-                    ],
-                    [
-                        (1 + s) * (1 - t) * (2 + r - s + t) * 0.125
-                        - (1 - r) * (1 + s) * (1 - t) * (2 - s + t) * 0.125,
-                        -(1 - r) * (1 - t) * (2 + r - s + t) * 0.125
-                        + (1 - r) * (1 + s) * (1 - t) * (2 + r + t) * 0.125,
-                        (1 - r) * (1 + s) * (2 + r - s + t) * 0.125
-                        - (1 - r) * (1 + s) * (1 - t) * (2 + r - s) * 0.125,
-                    ],
-                    #
-                    [
-                        (1 - s) * (1 + t) * (2 + r + s - t) * 0.125
-                        - (1 - r) * (1 - s) * (1 + t) * (2 + s - t) * 0.125,
-                        (1 - r) * (1 + t) * (2 + r + s - t) * 0.125
-                        - (1 - r) * (1 - s) * (1 + t) * (2 + r - t) * 0.125,
-                        -(1 - r) * (1 - s) * (2 + r + s - t) * 0.125
-                        + (1 - r) * (1 - s) * (1 + t) * (2 + r + s) * 0.125,
-                    ],
-                    [
-                        -(1 - s) * (1 + t) * (2 - r + s - t) * 0.125
-                        + (1 + r) * (1 - s) * (1 + t) * (2 + s - t) * 0.125,
-                        (1 + r) * (1 + t) * (2 - r + s - t) * 0.125
-                        - (1 + r) * (1 - s) * (1 + t) * (2 - r - t) * 0.125,
-                        -(1 + r) * (1 - s) * (2 - r + s - t) * 0.125
-                        + (1 + r) * (1 - s) * (1 + t) * (2 - r + s) * 0.125,
-                    ],
-                    [
-                        -(1 + s) * (1 + t) * (2 - r - s - t) * 0.125
-                        + (1 + r) * (1 + s) * (1 + t) * (2 - s - t) * 0.125,
-                        -(1 + r) * (1 + t) * (2 - r - s - t) * 0.125
-                        + (1 + r) * (1 + s) * (1 + t) * (2 - r - t) * 0.125,
-                        -(1 + r) * (1 + s) * (2 - r - s - t) * 0.125
-                        + (1 + r) * (1 + s) * (1 + t) * (2 - r - s) * 0.125,
-                    ],
-                    [
-                        (1 + s) * (1 + t) * (2 + r - s - t) * 0.125
-                        - (1 - r) * (1 + s) * (1 + t) * (2 - s - t) * 0.125,
-                        -(1 - r) * (1 + t) * (2 + r - s - t) * 0.125
-                        + (1 - r) * (1 + s) * (1 + t) * (2 + r - t) * 0.125,
-                        -(1 - r) * (1 + s) * (2 + r - s - t) * 0.125
-                        + (1 - r) * (1 + s) * (1 + t) * (2 + r - s) * 0.125,
-                    ],
-                    #
-                    [
-                        -2 * r * (1 - s) * (1 - t) * 0.25,
-                        -(1 - r) * (1 + r) * (1 - t) * 0.25,
-                        -(1 - r) * (1 + r) * (1 - s) * 0.25,
-                    ],
-                    [
-                        (1 - s) * (1 + s) * (1 - t) * 0.25,
-                        -2 * s * (1 + r) * (1 - t) * 0.25,
-                        -(1 - s) * (1 + s) * (1 + r) * 0.25,
-                    ],
-                    [
-                        -2 * r * (1 + s) * (1 - t) * 0.25,
-                        (1 - r) * (1 + r) * (1 - t) * 0.25,
-                        -(1 - r) * (1 + r) * (1 + s) * 0.25,
-                    ],
-                    [
-                        -(1 - s) * (1 + s) * (1 - t) * 0.25,
-                        -2 * s * (1 - r) * (1 - t) * 0.25,
-                        -(1 - s) * (1 + s) * (1 - r) * 0.25,
-                    ],
-                    #
-                    [
-                        -2 * r * (1 - s) * (1 + t) * 0.25,
-                        -(1 - r) * (1 + r) * (1 + t) * 0.25,
-                        (1 - r) * (1 + r) * (1 - s) * 0.25,
-                    ],
-                    [
-                        (1 - s) * (1 + s) * (1 + t) * 0.25,
-                        -2 * s * (1 + r) * (1 + t) * 0.25,
-                        (1 - s) * (1 + s) * (1 + r) * 0.25,
-                    ],
-                    [
-                        -2 * r * (1 + s) * (1 + t) * 0.25,
-                        (1 - r) * (1 + r) * (1 + t) * 0.25,
-                        (1 - r) * (1 + r) * (1 + s) * 0.25,
-                    ],
-                    [
-                        -(1 - s) * (1 + s) * (1 + t) * 0.25,
-                        -2 * s * (1 - r) * (1 + t) * 0.25,
-                        (1 - s) * (1 + s) * (1 - r) * 0.25,
-                    ],
-                    #
-                    [
-                        -(1 - t) * (1 + t) * (1 - s) * 0.25,
-                        -(1 - t) * (1 + t) * (1 - r) * 0.25,
-                        -2 * t * (1 - r) * (1 - s) * 0.25,
-                    ],
-                    [
-                        (1 - t) * (1 + t) * (1 - s) * 0.25,
-                        -(1 - t) * (1 + t) * (1 + r) * 0.25,
-                        -2 * t * (1 + r) * (1 - s) * 0.25,
-                    ],
-                    [
-                        (1 - t) * (1 + t) * (1 + s) * 0.25,
-                        (1 - t) * (1 + t) * (1 + r) * 0.25,
-                        -2 * t * (1 + r) * (1 + s) * 0.25,
-                    ],
-                    [
-                        -(1 - t) * (1 + t) * (1 + s) * 0.25,
-                        (1 - t) * (1 + t) * (1 - r) * 0.25,
-                        -2 * t * (1 - r) * (1 + s) * 0.25,
-                    ],
-                ]
-            )
-            * 0.125
+                    (1 - s) * (1 - t) * (2 + r + s + t) * 0.125
+                    - (1 - r) * (1 - s) * (1 - t) * 0.125,
+                    (1 - r) * (1 - t) * (2 + r + s + t) * 0.125
+                    - (1 - r) * (1 - s) * (1 - t) * 0.125,
+                    (1 - r) * (1 - s) * (2 + r + s + t) * 0.125
+                    - (1 - r) * (1 - s) * (1 - t) * 0.125,
+                ],
+                [
+                    -(1 - s) * (1 - t) * (2 - r + s + t) * 0.125
+                    + (1 + r) * (1 - s) * (1 - t) * 0.125,
+                    (1 + r) * (1 - t) * (2 - r + s + t) * 0.125
+                    - (1 + r) * (1 - s) * (1 - t) * 0.125,
+                    (1 + r) * (1 - s) * (2 - r + s + t) * 0.125
+                    - (1 + r) * (1 - s) * (1 - t) * 0.125,
+                ],
+                [
+                    -(1 + s) * (1 - t) * (2 - r - s + t) * 0.125
+                    + (1 + r) * (1 + s) * (1 - t) * 0.125,
+                    -(1 + r) * (1 - t) * (2 - r - s + t) * 0.125
+                    + (1 + r) * (1 + s) * (1 - t) * 0.125,
+                    (1 + r) * (1 + s) * (2 - r - s + t) * 0.125
+                    - (1 + r) * (1 + s) * (1 - t) * 0.125,
+                ],
+                [
+                    (1 + s) * (1 - t) * (2 + r - s + t) * 0.125
+                    - (1 - r) * (1 + s) * (1 - t) * 0.125,
+                    -(1 - r) * (1 - t) * (2 + r - s + t) * 0.125
+                    + (1 - r) * (1 + s) * (1 - t) * 0.125,
+                    (1 - r) * (1 + s) * (2 + r - s + t) * 0.125
+                    - (1 - r) * (1 + s) * (1 - t) * 0.125,
+                ],
+                #
+                [
+                    (1 - s) * (1 + t) * (2 + r + s - t) * 0.125
+                    - (1 - r) * (1 - s) * (1 + t) * 0.125,
+                    (1 - r) * (1 + t) * (2 + r + s - t) * 0.125
+                    - (1 - r) * (1 - s) * (1 + t) * 0.125,
+                    -(1 - r) * (1 - s) * (2 + r + s - t) * 0.125
+                    + (1 - r) * (1 - s) * (1 + t) * 0.125,
+                ],
+                [
+                    -(1 - s) * (1 + t) * (2 - r + s - t) * 0.125
+                    + (1 + r) * (1 - s) * (1 + t) * 0.125,
+                    (1 + r) * (1 + t) * (2 - r + s - t) * 0.125
+                    - (1 + r) * (1 - s) * (1 + t) * 0.125,
+                    -(1 + r) * (1 - s) * (2 - r + s - t) * 0.125
+                    + (1 + r) * (1 - s) * (1 + t) * 0.125,
+                ],
+                [
+                    -(1 + s) * (1 + t) * (2 - r - s - t) * 0.125
+                    + (1 + r) * (1 + s) * (1 + t) * 0.125,
+                    -(1 + r) * (1 + t) * (2 - r - s - t) * 0.125
+                    + (1 + r) * (1 + s) * (1 + t) * 0.125,
+                    -(1 + r) * (1 + s) * (2 - r - s - t) * 0.125
+                    + (1 + r) * (1 + s) * (1 + t) * 0.125,
+                ],
+                [
+                    (1 + s) * (1 + t) * (2 + r - s - t) * 0.125
+                    - (1 - r) * (1 + s) * (1 + t) * 0.125,
+                    -(1 - r) * (1 + t) * (2 + r - s - t) * 0.125
+                    + (1 - r) * (1 + s) * (1 + t) * 0.125,
+                    -(1 - r) * (1 + s) * (2 + r - s - t) * 0.125
+                    + (1 - r) * (1 + s) * (1 + t) * 0.125,
+                ],
+                #
+                [
+                    -2 * r * (1 - s) * (1 - t) * 0.25,
+                    -(1 - r) * (1 + r) * (1 - t) * 0.25,
+                    -(1 - r) * (1 + r) * (1 - s) * 0.25,
+                ],
+                [
+                    (1 - s) * (1 + s) * (1 - t) * 0.25,
+                    -2 * s * (1 + r) * (1 - t) * 0.25,
+                    -(1 - s) * (1 + s) * (1 + r) * 0.25,
+                ],
+                [
+                    -2 * r * (1 + s) * (1 - t) * 0.25,
+                    (1 - r) * (1 + r) * (1 - t) * 0.25,
+                    -(1 - r) * (1 + r) * (1 + s) * 0.25,
+                ],
+                [
+                    -(1 - s) * (1 + s) * (1 - t) * 0.25,
+                    -2 * s * (1 - r) * (1 - t) * 0.25,
+                    -(1 - s) * (1 + s) * (1 - r) * 0.25,
+                ],
+                #
+                [
+                    -2 * r * (1 - s) * (1 + t) * 0.25,
+                    -(1 - r) * (1 + r) * (1 + t) * 0.25,
+                    (1 - r) * (1 + r) * (1 - s) * 0.25,
+                ],
+                [
+                    (1 - s) * (1 + s) * (1 + t) * 0.25,
+                    -2 * s * (1 + r) * (1 + t) * 0.25,
+                    (1 - s) * (1 + s) * (1 + r) * 0.25,
+                ],
+                [
+                    -2 * r * (1 + s) * (1 + t) * 0.25,
+                    (1 - r) * (1 + r) * (1 + t) * 0.25,
+                    (1 - r) * (1 + r) * (1 + s) * 0.25,
+                ],
+                [
+                    -(1 - s) * (1 + s) * (1 + t) * 0.25,
+                    -2 * s * (1 - r) * (1 + t) * 0.25,
+                    (1 - s) * (1 + s) * (1 - r) * 0.25,
+                ],
+                #
+                [
+                    -(1 - t) * (1 + t) * (1 - s) * 0.25,
+                    -(1 - t) * (1 + t) * (1 - r) * 0.25,
+                    -2 * t * (1 - r) * (1 - s) * 0.25,
+                ],
+                [
+                    (1 - t) * (1 + t) * (1 - s) * 0.25,
+                    -(1 - t) * (1 + t) * (1 + r) * 0.25,
+                    -2 * t * (1 + r) * (1 - s) * 0.25,
+                ],
+                [
+                    (1 - t) * (1 + t) * (1 + s) * 0.25,
+                    (1 - t) * (1 + t) * (1 + r) * 0.25,
+                    -2 * t * (1 + r) * (1 + s) * 0.25,
+                ],
+                [
+                    -(1 - t) * (1 + t) * (1 + s) * 0.25,
+                    (1 - t) * (1 + t) * (1 - r) * 0.25,
+                    -2 * t * (1 - r) * (1 + s) * 0.25,
+                ],
+            ]
         )
 
 
-class Tri1(Tri):
+class TriQuadraticHexahedron(Hexahedron):
     def __init__(self):
         super().__init__()
-        self.nnodes = 3
+        self.npoints = 27
+        self.nbasis = 27
+        self.lagrange = ArbitraryOrderLagrange(order=2, ndim=3)
+        self.vertices = np.array([0, 2, 8, 6, 18, 20, 26, 24])
+        self.edges = np.array([1, 5, 7, 3, 19, 23, 25, 21, 9, 11, 17, 15])
+        self.faces = np.array([12, 14, 10, 16, 4, 22])
+        self.volume = np.array([13])
+        self.permute = np.concatenate(
+            (self.vertices, self.edges, self.faces, self.volume)
+        )
+
+    def basis(self, rst):
+        return self.lagrange.basis(rst)[self.permute]
+
+    def basisprime(self, rst):
+        return self.lagrange.basisprime(rst)[self.permute, :]
+
+
+class Triangle(TriangleElement):
+    def __init__(self):
+        super().__init__()
+        self.npoints = 3
         self.nbasis = 3
 
     def basis(self, rs):
@@ -442,10 +462,35 @@ class Tri1(Tri):
         return np.array([[-1, -1], [1, 0], [0, 1]], dtype=float)
 
 
-class Tet1(Tet):
+class TriangleMINI(TriangleElement):
     def __init__(self):
         super().__init__()
-        self.nnodes = 4
+        self.npoints = 4
+        self.nbasis = 3
+
+    def basis(self, rs):
+        "linear triangle basis functions"
+        r, s = rs
+        return np.array([1 - r - s, r, s, 27 * r * s * (1 - r - s)])
+
+    def basisprime(self, rs):
+        "linear triangle derivative of basis functions"
+        r, s = rs
+        return np.array(
+            [
+                [-1, -1],
+                [1, 0],
+                [0, 1],
+                [27 * (s * (1 - r - s) - r * s), 27 * (r * (1 - r - s) - r * s)],
+            ],
+            dtype=float,
+        )
+
+
+class Tetra(TetraElement):
+    def __init__(self):
+        super().__init__()
+        self.npoints = 4
         self.nbasis = 4
 
     def basis(self, rst):
@@ -459,10 +504,40 @@ class Tet1(Tet):
         return np.array([[-1, -1, -1], [1, 0, 0], [0, 1, 0], [0, 0, 1]], dtype=float)
 
 
-class Tri2(Tri):
+class TetraMINI(TetraElement):
     def __init__(self):
         super().__init__()
-        self.nnodes = 6
+        self.npoints = 5
+        self.nbasis = 4
+
+    def basis(self, rst):
+        "linear tetrahedral basis functions"
+        r, s, t = rst
+        return np.array([1 - r - s - t, r, s, t, 256 * r * s * t * (1 - r - s - t)])
+
+    def basisprime(self, rst):
+        "linear tetrahedral derivative of basis functions"
+        r, s, t = rst
+        return np.array(
+            [
+                [-1, -1, -1],
+                [1, 0, 0],
+                [0, 1, 0],
+                [0, 0, 1],
+                [
+                    256 * (s * t * (1 - r - s - t) - r * s * t),
+                    256 * (r * t * (1 - r - s - t) - r * s * t),
+                    256 * (r * s * (1 - r - s - t) - r * s * t),
+                ],
+            ],
+            dtype=float,
+        )
+
+
+class QuadraticTriangle(TriangleElement):
+    def __init__(self):
+        super().__init__()
+        self.npoints = 6
         self.nbasis = 6
 
     def basis(self, rs):
@@ -479,12 +554,17 @@ class Tri2(Tri):
     def basisprime(self, rs):
         "linear triangle derivative of basis functions"
         r, s = rs
-        dhdr_a = np.array([[-1, -1], [1, 0], [0, 1]])
+
+        t1 = 1 - r - s
+        t2 = r
+        t3 = s
+
+        dhdr_a = np.array([[-1, -1], [1, 0], [0, 1]], dtype=float)
         dhdr_b = np.array(
             [
-                [4 * (1 - r - s) - 4 * r, -4 * r],
-                [4 * s, 4 * r],
-                [-4 * s, 4 * (1 - r - s) - 4 * s],
+                [4 * (t1 - t2), -4 * t2],
+                [4 * t3, 4 * t2],
+                [-4 * t3, 4 * (t1 - t2)],
             ]
         )
         dhdr = np.vstack((dhdr_a, dhdr_b))
@@ -494,51 +574,62 @@ class Tri2(Tri):
         return dhdr
 
 
-class Tet2(Tet):
+class QuadraticTetra(TetraElement):
     def __init__(self):
         super().__init__()
-        self.nnodes = 10
+        self.npoints = 10
         self.nbasis = 10
 
     def basis(self, rst):
         "linear tetrahedral basis functions"
         r, s, t = rst
+
+        t1 = 1 - r - s - t
+        t2 = r
+        t3 = s
+        t4 = t
+
         h = np.array(
             [
-                1 - r - s - t,
-                r,
-                s,
-                t,
-                4 * r * (1 - r - s - t),
-                4 * r * s,
-                4 * s * (1 - r - s - t),
-                4 * r * t,
-                4 * s * t,
-                4 * t * (1 - r - s - t),
+                t1 * (2 * t1 - 1),
+                t2 * (2 * t2 - 1),
+                t3 * (2 * t3 - 1),
+                t4 * (2 * t4 - 1),
+                4 * t1 * t2,
+                4 * t2 * t3,
+                4 * t3 * t1,
+                4 * t1 * t4,
+                4 * t2 * t4,
+                4 * t3 * t4,
             ]
         )
-        h[0] += -h[4] / 2 - h[6] / 2 - h[9] / 2
-        h[1] += -h[4] / 2 - h[5] / 2 - h[7] / 2
-        h[2] += -h[5] / 2 - h[6] / 2 - h[8] / 2
-        h[3] += -h[7] / 2 - h[8] / 2 - h[9] / 2
+
+        return h
 
     def basisprime(self, rst):
         "linear tetrahedral derivative of basis functions"
         r, s, t = rst
-        dhdr_a = np.array([[-1, -1, -1], [1, 0, 0], [0, 1, 0], [0, 0, 1]])
-        dhdr_b = np.array(
+
+        t1 = 1 - r - s - t
+        t2 = r
+        t3 = s
+        t4 = t
+
+        dhdt = np.array(
             [
-                [4 * (1 - r - s - t) - 4 * r, -4 * r, -4 * r],
-                [4 * s, 4 * r, 0],
-                [-4 * s, 4 * (1 - r - s - t) - 4 * s, -4 * s],
-                [4 * t, 0, 4 * r],
-                [0, 4 * t, 4 * s],
-                [-4 * t, -4 * t, 4 * (1 - r - s - t) - 4 * t],
+                [4 * t1 - 1, 0, 0, 0],
+                [0, 4 * t2 - 1, 0, 0],
+                [0, 0, 4 * t3 - 1, 0],
+                [0, 0, 0, 4 * t4 - 1],
+                [4 * t2, 4 * t1, 0, 0],
+                [0, 4 * t3, 4 * t2, 0],
+                [4 * t3, 0, 4 * t1, 0],
+                [4 * t4, 0, 0, 4 * t1],
+                [0, 4 * t4, 0, 4 * t2],
+                [0, 0, 4 * t4, 4 * t3],
             ]
         )
-        dhdr = np.vstack((dhdr_a, dhdr_b))
-        dhdr[0] += -dhdr[4] / 2 - dhdr[6] / 2 - dhdr[9] / 2
-        dhdr[1] += -dhdr[4] / 2 - dhdr[5] / 2 - dhdr[7] / 2
-        dhdr[2] += -dhdr[5] / 2 - dhdr[6] / 2 - dhdr[8] / 2
-        dhdr[3] += -dhdr[7] / 2 - dhdr[8] / 2 - dhdr[9] / 2
-        return dhdr
+
+        dtdr = np.array([[-1, -1, -1], [1, 0, 0], [0, 1, 0], [0, 0, 1]], dtype=float)
+
+        return np.dot(dhdt, dtdr)
